@ Pour utiliser les interruptions, il faut faire quelques choses:
@ 1. Les activer (REG_IME bit 0)
@ 2. Choisir/activer un (ou plus) types d'interruptions dans REG_IE
@ 3. Charger l'adresse de notre interruption dans REG_INTADDR
@ 4. (Possiblement) dire au LCD que nous voulons qu'il provoque une interruption
@ 5. Armer le bit correspondant à notre interruption dans REG_IF pour dire que l'interruption est finie
@	 (cette partie va dans le code de notre interruption)

@ Ici nous voulons activer/installer une interruption de VBLANK
interruptInit:
	mov r0, #REG_BASE				@ Quelles interruptions voulons nous gérer ?
@ REG_INTADDR : Charger l'adresse de l'interruption
	adr r1, interrupt				@ Charger une addresse +/-4k
	str r1, [r0, #REG_INTADDR_OFF]	@ Maintenant l'adresse de notre interruption est chargée
@ REG_DISPSTAT : Déclencher interruptions VBLANK
	mov r1, #STAT_VBLANK_IRQ		@ 
	strh r1, [r0, #REG_DISPSTAT_OFF]

	add r0, #0x200					@ r0 pointe sur REG_IE
@ REG_IE : Activer interruptions VBLANK
	mov r1, #INT_VBLANK				@ Les interruptions pendant VBLANK (après finir afficher l'écran) (bit 0)
	strh r1, [r0]					@ r1 -> [REG_IE] REG_IE = interruption VBLANK
@ REG_IME : Activer interruptions
	mov r1, #INT_ENABLE				@ =1, armer bit 0
	str r1, [r0, #REG_IME_OFF]		@ REG_IME : Activer interruptions
	bx lr							@ Quitter

@ Le code de notre interruption
interrupt:
	mov r0, #REG_DISPCNT			@ Une petite optimisation, en lieu de "ldr r0,=REG_IE" on calcule l'offset de 0x0400:0000
	ldr r1, [r0,#REG_IE-REG_DISPCNT]	@ r1 = REG_IE (2 octets) et REG_IF (2 octets)
	and r0, r1,r1, LSR #16			@ ANDer REG_IE et REG_IF
	tst r0, #INT_VBLANK
	 bxeq lr						@ Si l'interruption n'est pas un VBLANK, quitter

@ Décaler l'écran
	ldr r0, =x
	ldr r1, =y
	ldr r0,[r0]						@ Charger X
	ldr r1,[r1]						@ Charger Y
	mov r0,r0,LSL #8				@ Premier 8 bits de REG_BG2X = fraction
	mov r1,r1,LSL #8
	ldr r3,=REG_BG2X
	str r0,[r3], #4					@ X
	str r1,[r3]						@ Y REG_BG2Y

@ Annoncer que l'on a fini son travail avec l'interruption
@ r0 = REG_IE (début des registres d'interruption)
@ r1 = REG_BASE (REG_DISPCNT)
	mov r0, #REG_BASE				@ 0x0400:0000 (REG_DISPCNT)
	add r0, #REG_INT_OFF			@ 0x0400:0200 (début des registres d'interruption)
	ldr r2, [r0]					@ r2 = REG_IE (2 octets) et REG_IF (2 octets)
	and r2, r2,r2, LSR #16			@ ANDer REG_IE et REG_IF
	ldrh r3, [r1, #-8]				@ Les drapeaux d'interruptions du BIOS (0x3FFFFF8 est un miroir de 0x3007FF8)
	orr r2, r2, r3					@ (REG_IE & REG_IF) | "BIOS_IF"
	mov r1, #REG_BASE				@ 0x0400:0000 (REG_DISPCNT)
	strh r2, [r1, #-8]				@ Les écrire (pour que le hardware les efface, il n'a pas de sens, je sais)
	strh r2, [r0, #REG_IF_OFF]	@ 
	bx lr
