.arm
#DEFINE ARM					@ Pour les functions BIOS
.include "gba.inc"
.text
.global main

main:
	bl interruptInit		@ Initialiser les interruptions (VBLANK) [interrupts.asm]
@ Préparer le fond
	mov r0, #REG_DISPCNT	@ Display Control Register
	mov r1, #BG2_ENABLE
	orr r1, #MODE_1			@ Mode 1 avec BG2
	str r1,[r0]

	ldr r0, =REG_BG2CNT		@ BG2 Control Register
	ldr r1, =ROTBG_SIZE_512x512|BG_COLOR_256|(16<<SCREEN_SHIFT)|(0<<CHAR_SHIFT)
							@ Screen Base Block Number :
							@ Où commence le tilemap, ici 0x6000000 + 16*0x800
							@ Char Base Block Number :
							@ Où commence le tiledata, ici 0x6000000 + 0*0x4000
	str r1, [r0]			@ Charger l'information dans le registre de BG2

@ Charger les sprites dans le CHAR BLOCK	
	ldr r1, =sprites
	ldr r0, =REG_DMA3SAD	@ Adresse de source
	str r1, [r0], #4		@ Charger l'addresse et avancer 4 octets
							@ Maintenant r0 = REG_DMA3DAD
	mov r1, #CHAR_BLOCK_0	@ = 0x6000000 (aligné, donc on peut utiliser mov)
	str r1, [r0], #4		@ Charger l'addresse de destination
							@ Maintenant r0 = REG_DMA3CNT (counter)
	mov r1, #DMA_ENABLE		@ Faire le DMA, valeurs par défaut : halfword et s'exécuter immédiatement
	orr r1, r1, #(32*32)*32/4	@ taille = un sprite 32x32, mais on l'affiche quatre octets à la fois
	str r1, [r0]			@ Faire le DMA
@ Peut-être l'on /\ peut y ajouter #-8 ?

@ Charger la palette dans VPAL
	ldr r1, =palette
	ldr r0, =REG_DMA3SAD	@ Adresse de source
	str r1, [r0], #4		@ Charger l'addresse et avancer 4 octets
							@ Maintenant r0 = REG_DMA3DAD
	mov r1, #VPAL			@ = 0x05000000 (aligné)
	str r1, [r0], #4		@ Charger l'addresse de destination
							@ Maintenant r0 = REG_DMA3CNT (counter)
	mov r1, #DMA_32NOW		@ Faire le DMA avec des words (32 bits)
	orr r1, r1, #128		@ 256 couleurs, chaqu'un occupe 2 octets, transférer avec des words (4 octets)
	str r1, [r0]			@ Faire le DMA

@ Charger les tuiles (le map) dans le SCREEN BLOCK
	bl drawMap
@	ldr r1, =map
@	ldr r0, =REG_DMA3SAD	@ Adresse de source
@	str r1, [r0], #4		@ Charger l'addresse et avancer 4 octets
@							@ Maintenant r0 = REG_DMA3DAD
@	ldr r1, =SCREEN_BLOCK_28
@	str r1, [r0], #4		@ Charger l'addresse de destination
@							@ Maintenant r0 = REG_DMA3CNT (counter)
@	mov r1, #DMA_32NOW		@ Faire le DMA avec des words (32 bits)
@	orr r1, r1, #((32/8)*16)*((32/8)*16)		@ Combien d'octets à copier
@	str r1, [r0]			@ Faire le DMA

dontQuit:
	bl getKey
@ Quelle touche a-t-on poussée ?
	ldr r0, =x
	ldr r1, =y

	tst r4, #KEY_UP
	bleq keyUp
	tst r4, #KEY_DOWN
	bleq keyDown
	tst r4, #KEY_RIGHT
	bleq keyRight
	tst r4, #KEY_LEFT
	bleq keyLeft


	mov r0,#0x1200
delayLoop:
	subs r0,r0,#1
	bne delayLoop
	b dontQuit

keyUp:
	ldrh r2, [r1]			@ Charger octet (Y) à [r1] dans r2
	subs r2, r2, #1			@ Y-1
	bxcc lr					@ Si Y-1 arme le drapeau C, quitter
	strh r2, [r1]			@ Sauver nouvelle coordonnée
	bx lr					@ Quitter la routine
keyDown:
	ldrh r2, [r1]			@ Y
	add r2, r2, #1			@ Y+1
	mov r3, #0x200
	sub r3, #160			@ Hauteur de carte - hauteur d'écran
	cmp r2, r3				@ A-t-on atteint la fin de la carte ?
	bxcs lr
	strh r2, [r1]			@ Sauver nouvelle coordonnée
	bx lr					@ Quitter la routine
keyRight:
	ldrh r2, [r0]			@ X
	add r2, r2, #1			@ X+1
	mov r3, #0x200
	sub r3, #240			@ Largeur de carte - largeur d'écran
	cmp r2, r3				@ A-t-on atteint la fin de la carte ?
	bxcs lr
	strh r2, [r0]			@ Sauver nouvelle coordonnée
	bx lr					@ Quitter la routine
keyLeft:
	ldrh r2, [r0]			@ X
	subs r2, r2, #1			@ X-1
	bxcc lr
	strh r2, [r0]			@ Sauver nouvelle coordonnée
	bx lr					@ Quitter la routine

getKey:
	mov r3, #KEYS_HIGH		@ MSB
	orr r3, r3, #KEYS_LOW	@ Ajouter LSB de l'adresse
	mov r2, #0x300
	orr r2, #0xFF			@ r2 = $3FF
getKeyLoop:
	ldr r4, [r3]			@ Lire les touches poussées
@	teq r4, r2				@ Si nous n'avons pas appuyé sur une touche
@	beq getKeyLoop			@ ... répéter
	bx lr					@ Quitter la routine

.include "drawMap32.asm"
.include "interrupts.asm"

.ltorg

sprites:
.incbin "tilemap.bin"
palette:
.incbin "tilemap.pal"

map:						@ 16x16 (tuiles : 32x32 pixels)
.byte 0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0	@ 0
.byte 2,2,2,2,1,2,2,2,2,2,2,2,2,2,2,2	@ 1
.byte 2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2	@ 2
.byte 2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2	@ 3
.byte 2,2,2,2,2,2,1,2,2,2,2,2,2,2,2,2	@ 4
.byte 2,2,2,2,1,3,2,2,1,2,2,2,2,2,2,2	@ 5
.byte 2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2	@ 6
.byte 2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2	@ 7
.byte 2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2	@ 8
.byte 2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2	@ 9
.byte 0,0,0,0,0,2,0,0,1,2,1,2,1,2,2,2	@ 10
.byte 2,0,0,0,0,0,0,0,1,2,1,2,2,2,2,2	@ 11
.byte 2,2,2,2,2,2,2,2,1,1,1,2,1,2,2,2	@ 12
.byte 2,2,2,2,2,2,2,2,1,2,1,2,1,2,2,2	@ 13
.byte 2,2,2,2,2,2,2,2,1,2,1,2,1,2,2,2	@ 14
.byte 2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2	@ 15

.align 4
tile32x32:					@ La GBA seulement peut gérer les tuiles 8x8
tile0:
.byte 0x0,0x1,0x2,0x3
.byte 0x4,0x5,0x6,0x7
.byte 0x8,0x9,0xA,0xB
.byte 0xC,0xD,0xE,0xF
tile1:
.byte 0x10,0x11,0x12,0x13
.byte 0x14,0x15,0x16,0x17
.byte 0x18,0x19,0x1A,0x1B
.byte 0x1C,0x1D,0x1E,0x1F
tile2:
.byte 0x20,0x21,0x22,0x23
.byte 0x24,0x25,0x26,0x27
.byte 0x28,0x29,0x2A,0x2B
.byte 0x2C,0x2D,0x2E,0x2F
tile3:
.byte 0x30,0x31,0x32,0x33
.byte 0x34,0x35,0x36,0x37
.byte 0x38,0x39,0x3A,0x3B
.byte 0x3C,0x3D,0x3E,0x3F

.data
x: .word 0
y: .word 0
